<?php

namespace common\logic\Distribution\Distribution\Repository;

use common\logic\AbstractRepository;
use common\logic\Distribution\Distribution\Model\Distribution;
use common\logic\Distribution\Distribution\Service\DistributionSolver;
use common\logic\RepositoryQueryInterface;
use common\logic\Subscription\Subscription\Model\Subscription;
use common\logic\Subscription\Subscription\Service\SubscriptionSolver;

class DistributionRepository extends AbstractRepository
{
    protected DistributionRepositoryQuery $query;
    protected DistributionSolver $distributionSolver;
    protected SubscriptionSolver $subscriptionSolver;

    public function loadDependencies(): void
    {
        $this->loadQuery(DistributionRepositoryQuery::class);
        $this->distributionSolver = $this->loadService(DistributionSolver::class);
        $this->subscriptionSolver = $this->loadService(SubscriptionSolver::class);
    }

    public function getDefaultOptionsSearch(): array
    {
        return [
            self::WITH => [],
            self::JOIN_WITH => [],
            self::ORDER_BY => 'date ASC',
            self::ATTRIBUTE_ID_PRODUCER => 'distribution.id_producer'
        ];
    }

    public function findOneDistributionById(int $id): ?Distribution
    {
        return $this->createQuery()
            ->filterById($id)
            ->findOne();
    }

    public function findOneDistribution(string $date, bool $active = null): ?Distribution
    {
        $query = $this->createDefaultQuery();
        $query->filterByDate($date);
        if(!is_null($active)) {
            $query->filterByActive($active);
        }
        return $query->findOne();
    }

    public function findOneFirstDistribution(): ?Distribution
    {
        return $this->createDefaultQuery()
            ->orderBy('date ASC')
            ->findOne();
    }

    public function findOneLastDistribution(): ?Distribution
    {
        return $this->createDefaultQuery()
            ->orderBy('date DESC')
            ->findOne();
    }

    public function findDistributionsActive(): array
    {
        return $this->createDefaultQuery()
            ->filterByActive(true)
            ->orderBy('date ASC')
            ->find();
    }

    /**
     * Récupère les distributions futures.
     */
    public function findDistributionsIncoming($ignoreActive = false): array
    {
        $query = $this->createDefaultQuery();

        if(!$ignoreActive) {
            $query->filterByActive(true);
        }

        $query
            ->filterByDateBetterThanToday()
            ->orderBy('date ASC');

        return $query->find();
    }

    /**
     * Recherche les distributions futures où l'abonnement peut s'appliquer.
     */
    public function findDistributionsIncomingMatchWithSubscrtiption(Subscription $subscription, bool $filterByOrderDeadline = false): array
    {
        $incomingDistributionsArray = $this->createDefaultQuery()
            ->filterByDateBetterThanToday()
            ->filterBySubscriptionDates($subscription)
            ->find();

        if($filterByOrderDeadline) {
            $incomingDistributionsArray = $this->distributionSolver
                ->filterDistributionsByDateDelay($incomingDistributionsArray);
        }

        $matchedIncomingDistributionsArray = [];
        foreach ($incomingDistributionsArray as $incomingDistribution) {
            if ($this->subscriptionSolver->isSubscriptionMatchWith($subscription, $incomingDistribution->date)) {
                $matchedIncomingDistributionsArray[] = $incomingDistribution;
            }
        }

        return $matchedIncomingDistributionsArray;
    }

    public function isDistributionDateAvailable(string $date = null): bool
    {
        if($date && strlen($date)) {
            $distribution = $this->findOneDistribution($date);
            if($distribution && $this->distributionSolver->isDistributionAvailable($distribution)) {
                return true;
            }
        }
        
        return false;
    }
}